//
// Copyright 2020-2021 Signal Messenger, LLC.
// SPDX-License-Identifier: AGPL-3.0-only
//

#![allow(clippy::missing_safety_doc)]
#![deny(clippy::unwrap_used)]

#[cfg(not(any(feature = "ffi", feature = "jni", feature = "node")))]
compile_error!("Feature \"ffi\", \"jni\", or \"node\" must be enabled for this crate.");

pub use libsignal_bridge_types::{
    bridge_as_handle, bridge_deserialize, bridge_fixed_length_serializable_fns, bridge_get,
    bridge_handle_fns, bridge_serializable_handle_fns, describe_panic, io, support,
};

#[cfg(feature = "node")]
pub use libsignal_bridge_types::node_register;
#[cfg(feature = "ffi")]
pub use libsignal_bridge_types::{ffi_arg_type, ffi_result_type};
#[cfg(feature = "jni")]
pub use libsignal_bridge_types::{jni_arg_type, jni_args, jni_class_name, jni_result_type};

#[cfg(feature = "ffi")]
pub use libsignal_bridge_types::ffi;

#[cfg(feature = "jni")]
pub use libsignal_bridge_types::jni;

#[cfg(feature = "node")]
pub use libsignal_bridge_types::node;

pub mod logging;

pub mod crypto;
pub mod protocol;

// Desktop does not make use of device transfer certificates
#[cfg(any(feature = "jni", feature = "ffi"))]
pub mod device_transfer;

mod cds2;
mod hsm_enclave;
mod sgx_session;

pub mod zkgroup;

#[cfg(feature = "ffi")]
pub mod ias;

pub mod net;

// Desktop does not use SVR
#[cfg(any(feature = "jni", feature = "ffi"))]
mod pin;
#[cfg(any(feature = "jni", feature = "ffi"))]
mod svr2;

pub mod incremental_mac;
pub mod message_backup;
pub mod usernames;

#[cfg(feature = "signal-media")]
pub mod media;

// These APIs are only useful for tests. To save on code size, we omit them by default.
// To run tests, build with `--features testing-fns`.
#[cfg(feature = "testing-fns")]
pub mod testing;
